@@ -25,6 +25,7 @@ struct _XAppSnWatcher
     GHashTable *items;
 
     gboolean shutdown_pending;
+    gboolean advertise_host;
 };
 
 G_DEFINE_TYPE (XAppSnWatcher, xapp_sn_watcher, GTK_TYPE_APPLICATION)
@@ -66,7 +67,7 @@ handle_status_applet_name_owner_appeared (XAppSnWatcher *watcher,
             else
             {
                 sn_watcher_interface_set_is_status_notifier_host_registered (watcher->skeleton,
-                                                                             TRUE);
+                                                                             watcher->advertise_host);
                 g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (watcher->skeleton));
                 sn_watcher_interface_emit_status_notifier_host_registered (watcher->skeleton);
             }
@@ -204,9 +205,9 @@ on_name_acquired (GDBusConnection *connection,
     XAppSnWatcher *watcher = XAPP_SN_WATCHER (user_data);
 
     g_debug ("Name acquired on dbus");
-
+    sn_watcher_interface_set_protocol_version (watcher->skeleton, 0);
     sn_watcher_interface_set_is_status_notifier_host_registered (watcher->skeleton,
-                                                                 TRUE);
+                                                                 watcher->advertise_host);
     g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (watcher->skeleton));
     sn_watcher_interface_emit_status_notifier_host_registered (watcher->skeleton);
 }
@@ -295,6 +296,55 @@ create_key (const gchar  *sender,
     return TRUE;
 }
 
+typedef struct
+{
+    XAppSnWatcher *watcher;
+    gchar *key;
+    gchar *path;
+    gchar *bus_name;
+    gchar *service;
+} NewSnProxyData;
+
+static void
+sn_item_proxy_new_completed (GObject      *source,
+                             GAsyncResult *res,
+                             gpointer      user_data)
+{
+    NewSnProxyData *data = (NewSnProxyData *) user_data;
+    XAppSnWatcher *watcher = data->watcher;
+    SnItem *item;
+    GError *error = NULL;
+
+    SnItemInterface *proxy;
+
+    proxy = sn_item_interface_proxy_new_finish (res, &error);
+
+    if (error != NULL)
+    {
+        g_debug ("Could not create new status notifier proxy item for item at %s: %s",
+                 data->bus_name, error->message);
+        return;
+    }
+
+    item = sn_item_new ((GDBusProxy *) proxy,
+                        g_str_has_prefix (data->path, APPINDICATOR_PATH_PREFIX));
+
+    g_hash_table_insert (watcher->items,
+                         g_strdup (data->key),
+                         item);
+
+    update_published_items (watcher);
+
+    sn_watcher_interface_emit_status_notifier_item_registered (watcher->skeleton,
+                                                               data->service);
+
+    g_free (data->key);
+    g_free (data->path);
+    g_free (data->bus_name);
+    g_free (data->service);
+    g_slice_free (NewSnProxyData, data);
+}
+
 static gboolean
 handle_register_item (SnWatcherInterface     *skeleton,
                       GDBusMethodInvocation  *invocation,
@@ -304,7 +354,7 @@ handle_register_item (SnWatcherInterface     *skeleton,
     SnItem *item;
     GError *error;
     const gchar *sender;
-    g_autofree gchar *key, *bus_name, *path;
+    g_autofree gchar *key = NULL, *bus_name = NULL, *path = NULL;
 
     sender = g_dbus_method_invocation_get_sender (invocation);
 
@@ -322,40 +372,27 @@ handle_register_item (SnWatcherInterface     *skeleton,
 
     if (item == NULL)
     {
-        SnItemInterface *proxy;
+        NewSnProxyData *data;
         error = NULL;
         g_debug ("Key: '%s'", key);
 
-        proxy = sn_item_interface_proxy_new_sync (watcher->connection,
-                                                  G_DBUS_PROXY_FLAGS_NONE,
-                                                  bus_name,
-                                                  path,
-                                                  NULL,
-                                                  &error);
+        data = g_slice_new0 (NewSnProxyData);
+        data->watcher = watcher;
+        data->key = g_strdup (key);
+        data->path = g_strdup (path);
+        data->bus_name = g_strdup (bus_name);
+        data->service = g_strdup (service);
 
-        if (error != NULL)
-        {
-            g_debug ("Could not create new status notifier proxy item for item at %s: %s", bus_name, error->message);
-
-            g_dbus_method_invocation_return_gerror (invocation, error);
-
-            return FALSE;
-        }
-
-        item = sn_item_new ((GDBusProxy *) proxy,
-                            g_str_has_prefix (path, APPINDICATOR_PATH_PREFIX));
-
-        g_hash_table_insert (watcher->items,
-                             g_strdup (key),
-                             item);
-
-        update_published_items (watcher);
-
-        sn_watcher_interface_emit_status_notifier_item_registered (skeleton,
-                                                                   service);
+        sn_item_interface_proxy_new (watcher->connection,
+                                     G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
+                                     bus_name,
+                                     path,
+                                     NULL,
+                                     sn_item_proxy_new_completed,
+                                     data);
     }
 
-    sn_watcher_interface_complete_register_status_notifier_item (skeleton,
+    sn_watcher_interface_complete_register_status_notifier_item (watcher->skeleton,
                                                                  invocation);
 
     return TRUE;
@@ -475,6 +512,8 @@ watcher_startup (GApplication *application)
         g_application_quit (application);
     }
 
+    watcher->advertise_host = g_settings_get_boolean (xapp_settings, ADVERTISE_SNH_KEY);
+
     add_name_listener (watcher);
 
     if (xapp_status_icon_any_monitors ())
@@ -507,6 +546,7 @@ watcher_shutdown (GApplication *application)
         watcher->name_listener_id = 0;
     }
 
+    update_published_items (watcher);
     g_clear_pointer (&watcher->items, g_hash_table_unref);
 
     sn_watcher_interface_set_is_status_notifier_host_registered (watcher->skeleton,
@@ -514,8 +554,6 @@ watcher_shutdown (GApplication *application)
     g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (watcher->skeleton));
     sn_watcher_interface_emit_status_notifier_host_registered (watcher->skeleton);
 
-    update_published_items (watcher);
-
     if (watcher->owner_id > 0)
     {
         g_bus_unown_name (watcher->owner_id);
